#!/bin/bash

PACKAGES=
RUBY_VERSION=2.3.0
ABSOLUTE_PATH="$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)"

#Checa se o usuário é root
if [[ $EUID -ne 0 ]]
then
    ROOT="sudo"
else
    ROOT=
fi

#Descobre a distribuição
declare -A osInfo;
osInfo[/etc/redhat-release]=dnf
osInfo[/etc/arch-release]=pacman
osInfo[/etc/SuSE-release]=zypper
osInfo[/etc/debian_version]=apt-get

for f in ${!osInfo[@]}
do
    if [[ -f $f ]];then
        DISTRO=${osInfo[$f]}
    fi
done

#Ruby precisa de uma função especial para ser instalado através do RVM
function ruby {
    gpg --keyserver hkp://keys.gnupg.net --recv-keys 409B6B1796C275462A1703113804BB82D39DC0E3
    curl -sSL https://get.rvm.io | bash -s stable
    source $HOME/.rvm/scripts/rvm
    rvm install $RUBY_VERSION
}

function update {
    $(echo "$ROOT $DISTRO update")
    $(echo "$ROOT $DISTRO upgrade -y")
}

#Instala todos os pacotes no arquivo packages
function install {
    PACKAGES=$(cat "$ABSOLUTE_PATH/packages.txt" | cut -d: -f2 | tr '\r\n' ' ')
}

function fresh-install {
    update
    install
    $(echo "$ROOT $DISTRO install -y $PACKAGES")
    ruby
}

function list {
    cat "$ABSOLUTE_PATH/packages.txt" | tr ':' '\t'
}

function help {
    echo "$(basename "$0") [-f|--fresh-install] [-h|--help] [-l|--list] (pacotes) -- Utilize para instalar pacotes na sua máquina"
    echo
    echo "onde:"
    echo "      -h mostra a ajuda e sai"
    echo "      -f instala todos os pacotes da lista"
    echo "      -l lista os pacotes disponíveis"
    echo "exemplo de uso:"
    echo "      $(basename "$0") php mysql ruby"
    echo "AVISO: por padrão, o ruby não é instalado. Para instalar, execute $(basename "$0") ruby"
}

#Se o arquivo não existe
if [ ! -f "$ABSOLUTE_PATH/packages.txt" ]
then
  echo "AVISO: Lista de pacotes não encontrada! Adicione no mesmo diretório deste script o arquivo packages.txt."
  exit 0
fi


while [[ $# > 0 ]]
do
    key="$1"
    case $key in
        -f|--fresh-install)
        fresh-install
        exit 0
        ;;
        -h|--help)
        help
        exit 0
        ;;
        -l|--list)
        list
        exit 0
        ;;
        ruby)
        ruby
        ;;
        *)
        #concatena na lista de pacotes as regras passadas por parâmetro
        PACKAGES="$(cat "$ABSOLUTE_PATH/packages.txt" | grep $key | cut -d: -f2) $PACKAGES"
        ;;
    esac
    shift
done

#Checa se a string não está vazia
if [ ! -z "${PACKAGES// }" ]
then
    $(echo "$ROOT $DISTRO install -y $PACKAGES")
else
    help
fi
